#!/usr/local/bin/perl

# (my) Emacs outline to docbook very lite SGML 
# document. Should also parse under docbook.
# Bob DuCharme 4/15/97
# TODO: cover numbered lists
#


sub OpenElement;
sub CloseElement;
sub CloseSect;      # Close all non-sect elements.
sub CurrentElement; # Return GI of current element.

sub IsOpen;         # whether an element is open.
sub lower;          # Convert to lower case.

# You could comment out the first one instead if you like. 
$DOCTYPEdecl = "<!DOCTYPE chapter SYSTEM \"dbvlite.dtd\">";
# $DOCTYPEdecl = "<!DOCTYPE chapter PUBLIC \"-//Davenport//DTD DocBook V3.0//EN\">";

$LastLineBlank = 0; 

push (@OpenElements,"chapter");    # A stack.

print $DOCTYPEdecl;

OpenElement("chapter");
OpenElement("title");
print $ARGV[0];
CloseElement("title");
print "\n";

while (<>) {
	 s/</&lt;/g;
    s/&lt;emphasis>/<emphasis>/ig;
    s/&lt;\/emphasis>/<\/emphasis>/ig;
	 s/--/&mdash;/g;

	 if (/^\*\*\*\*/) {
		  s/^\*\*\*\*\s*//;
		  CloseSect;
		  CloseElement("sect4");  # If open.
		  OpenElement("sect4"); 
		  OpenElement("title");
		  chop($_);
		  print;
		  CloseElement("title");
		  print "\n";
	 }
	 elsif (/^\*\*\*/) {
		  s/^\*\*\*\s*//;
		  CloseSect;
		  CloseElement("sect3");  # If open.
		  OpenElement("sect3"); 
		  OpenElement("title");
		  chop($_);
		  print;
		  CloseElement("title");
		  print "\n";
	 }
	 elsif (/^\*\*/) {
		  s/^\*\*\s*//;
		  CloseSect;
		  CloseElement("sect3");
		  CloseElement("sect2");
		  OpenElement("sect2"); 
		  OpenElement("title");
		  print;
		  CloseElement("title");
		  print "\n";
	 }
	 elsif (/^\*/) {
		  s/^\*\s*//;
		  CloseSect;
		  CloseElement("sect3");
		  CloseElement("sect2");
		  CloseElement("sect1");
		  OpenElement("sect1"); 
		  OpenElement("title");
		  print;
		  CloseElement("title");
		  print "\n";
	 }
	 elsif (/^\-\s+/) {   # If the line starts with a hyphen and blanks.
		  if (IsOpen("itemizedlist") == 0) {
				CloseElement("para");        # If necessary.
				OpenElement("itemizedlist");
		  }
		  s/^-\s+//;
		  CloseElement("listitem");   # If one was open.
		  OpenElement("listitem");
		  OpenElement("para");
		  print;
	 }
	 elsif (IsOpen("itemizedlist")) {
		  if (/^\s*$/) {    # if it's a blank line
				CloseElement("para");
				OpenElement("para");
		  }
		  elsif (/^\s+/) {   # If a b list line begins with a space (not a
				# hyphen) followed by text, treat it as another paragraph 
				# of the previous list item.
				CloseElement("para");
				OpenElement("para");
				s/^\s+//;
				print;
		  }
		  else { # The line begins with plain text.
				if ($LastLineBlank == 1) { # End of list, treat it as plain text.
					 CloseElement("itemizedlist");   
					 OpenElement("para");
					 print;
				}
				elsif (CurrentElement() eq "para") {  # It's a continuation of the
					 print;                            # previous list item para.
				}
				else {
					 print;
				}
		  }
	 }  # So we're not in a list
	 elsif (/^\s*$/) {   # Blank line.
		  if (CurrentElement() eq "para") {
				CloseElement("para");
				OpenElement("para");
				print;     
		  }

	 }
	 else {
		  if (CurrentElement() ne "para") {
				OpenElement("para");            # Print as a regular paragraph.
		  } 
		  print; 
	 }

	 if (/^\s*$/) {
		  $LastLineBlank = 1;
	 }
	 else {
		  $LastLineBlank = 0;
	 }

} # end of main while loop

CloseElement("chapter");

######## subroutines ########

sub OpenElement {
	 local($NewElement) = lower(@_);
	 print "\n<$NewElement>";
	 push (@OpenElements,$NewElement);
}

sub IsOpen {            # Return 1 if element is open, else 0.
	 local($GI) = @_;
	 $FoundIt = 0;
	 foreach(@OpenElements) {    
		  $FoundIt = 1 if ($_ eq $GI);    # THIS SHOULDN'T KEEP SEARCHING
	 }
	 return $FoundIt;
}

sub CloseElement {
	 local($EndingElement) = @_;
	 if (&IsOpen($EndingElement)) {
		  while (CurrentElement() ne $EndingElement) {
				$CurrentOne = pop(@OpenElements);
				print "</$CurrentOne>";
		  }
		  $CurrentOne = pop(@OpenElements);   # Now get the target one.
		  print "</$CurrentOne>";
	 }  # else don't do anything. 
}


sub CloseSect {   # Close all non-sect elements.
	 $NextOne = CurrentElement();  # Element on top of stack.
	 while ((substr($NextOne,0,4) ne "sect") && ($NextOne ne "chapter")) {
		  $CurrentOne = pop(@OpenElements);
		  print "</$CurrentOne>";
		  $NextOne = CurrentElement();
	 }
}


sub lower {   # Convert to lower case.
	 local($string) = @_;
	 $string =~  s/(.*)/\L$1\E/g;
	 return $string;
}


sub CurrentElement { # Return GI of current element.
	 return $OpenElements[@OpenElements - 1];
}

##### eof #######

